import {Gamesystem} from "./gamesystems/Gamesystem";
import {ScriptAccount} from "./scriptAccounts/ScriptAccount";
import {Transition} from "./gamesystems/Transition";
import {State} from "./gamesystems/State";
import {ProductGamesystem} from "./gamesystems/ProductGamesystem";

export class GameModel {
  private readonly _gameModelName: string

  private _gamesystems: Gamesystem<any, any>[] = [];
  private _scriptAccounts: ScriptAccount[] = [];

  constructor(gameModelName: string) {
    this._gameModelName = gameModelName;
  }
  get gameModelName(): string {
    return this._gameModelName;
  }
  get gamesystems(): Gamesystem<any, any>[] {
    return this._gamesystems;
  }
  get scriptAccounts(): ScriptAccount[] {
    return this._scriptAccounts;
  }

  addGamesystem(gamesystem: Gamesystem<any, any>) {
    if(!this.gamesystems.includes(gamesystem)) {
      this._gamesystems.push(gamesystem);
    }
  }

  removeGamesystem(gamesystem : Gamesystem<any, any>) {
    this._gamesystems = this._gamesystems.filter(g => g !== gamesystem);
  }

  addScriptAccount(scriptAccountName: string) {
    if(scriptAccountName != undefined && scriptAccountName.length > 0) {
      const scriptAccount = new ScriptAccount(scriptAccountName, "");
      const searchedScriptAccount = this.scriptAccounts.find(s => s.componentName === scriptAccount.componentName);
      if(searchedScriptAccount == undefined) {
        this.scriptAccounts.push(scriptAccount);
        return scriptAccount;
      }
    }
    return undefined;
  }

  removeScriptAccount(scriptAccount: ScriptAccount) {
    if(scriptAccount != undefined) {
      this._scriptAccounts = this.scriptAccounts.filter(s => s != scriptAccount);
    }
  }

  findGamesystem(gamesystemName: string) {
    const gamesystemQueue : Gamesystem<State<any>, Transition<any>>[] = [];
    this.gamesystems.forEach(gamesystem => gamesystemQueue.push(gamesystem));

    while(gamesystemQueue.length > 0) {
      const currentGamesystem = gamesystemQueue.shift()!;
      if(currentGamesystem.componentName == gamesystemName) {
        return currentGamesystem;
      } else if(currentGamesystem instanceof ProductGamesystem) {
        const currentProductGamesystem = currentGamesystem as ProductGamesystem;
        currentProductGamesystem.innerGamesystems.forEach(gamesystem => gamesystemQueue.push(gamesystem));
      }
    }
  }

}
